#!/bin/bash

# bash-style patching for Liu & Li's mapmaking routines in GDL
#
# Usage:   LLmapmaking_apply_patches.sh
#
# This assumes that LLmapmaking_setup_env.sh has already been run
# to evaluate environment variables.

# Usage: LLmapmaking_apply_patches.sh       # a dry run by default
# Usage: LLmapmaking_apply_patches.sh yes   # when you are sure you want to do it

if [ "x$1" == "xyes" ] ; then
    DRY_RUN=""
else
    DRY_RUN="--dry-run"
    echo "Dry run of patches. Use parameter 'yes' to really apply patches."
fi

VERBOSE="--verbose"

# FIXED: 
# This protection against a GDL bug has been added upstream in the following version:
# http://idlastro.gsfc.nasa.gov/ftp/pro/fits/fits_info.pro
# -rw-r--r-- 11711 Apr  9 21:26 pro/fits_info.pro
# md5sum: 5edebf9b2b4d94640cc700814e39e750  pro/fits_info.pro
# It is not in  astron.dir.gz of Apr 7 2010, for the obvious reason.
#
# The patch here should give you the 9 April 2010 version of fits_info.pro.
# 
# GDL point_lun bug for compressed files, due to feature not yet being implemented
# in gzstream  http://ln-s.net/5h5n
#
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/fits_info.pro  \
    ${IDLASTRO_TOP}/pro/fits/fits_info.pro fits_info_20100412a.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/fits_info.pro && echo ""
fi


# idlastro:: fits_read: /integer missing in product() in  GDL-0.9~rc4
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/fits_read.pro  \
    ${IDLASTRO_TOP}/pro/fits/fits_read.pro fits_read_20100329.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/fits_read.pro && echo "" 
fi

# idlastro:: ftab_ext:   scope_varfetch missing in  GDL-0.9~rc4 
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/ftab_ext.pro  \
    ${IDLASTRO_TOP}/pro/fits_table/ftab_ext.pro ftab_ext_20100330a.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/ftab_ext.pro && echo ""
fi

# idlastro:: fxbwritem: needs to call SWAPxENDIANxINPLACE 
# Portability warning: this patch assumes that your machine is litte-endian.
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/fxbwritm.pro  \
    ${IDLASTRO_TOP}/pro/fits_bintable/fxbwritm.pro fxbwritm_20100330a.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/fxbwritm.pro && echo ""
fi

# mapmaking: main LL procedures, versus LLmapmaking v1
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/mapmaking.pro  \
    ${LLMAP_PRO}/mapmaking.pro mapmaking_20100422a.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/mapmaking.pro && echo ""
fi

# tod_unpack_quick: main LL procedures, versus LLmapmaking v1
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/tod_unpack_quick.pro  \
    ${LLMAP_PRO}/tod_unpack_quick.pro tod_unpack_quick_20100412a.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/tod_unpack_quick.pro && echo ""
fi



if [ "x$1" == "xyes" ] ; then
    echo ""
else
    echo "That used the patch option " ${DRY_RUN} ". Use parameter 'yes' to really apply patches."
fi



exit 0


##### The following patches are no longer needed if you use recent enough versions
##### of the libraries.

# FIXED: 
# This bug has been fixed upstream in the following version:
# astron.dir.tar.gz       07-Apr-2010 15:32  1.1M  
# -rw-r--r--  6154 Apr  5 16:52 fxbcreate.pro
# md5sum: d66ec3037ff69aa2a2a35e8f145a2583
# http://idlastro.gsfc.nasa.gov/ftp/pro/fits_bintable/fxbcreate.pro
#
# idlastro:: fxbcreate bug. fits_bintable/changes.txt 2009-02-10 
# fxbcreate 2002-12-13: adds THEAP to header and removes END statement
patch ${DRY_RUN} ${VERBOSE} -o ${LLMAP_GDL}/pro/fxbcreate.pro  \
    ${IDLASTRO_TOP}/pro/fits_bintable/fxbcreate.pro fxbcreate_20100402.patch
if [ "x$1" == "xyes" ] ; then 
    echo "patched file=" ${LLMAP_GDL}/pro/fxbcreate.pro && echo ""
fi

