#!/bin/bash

# bash setup of environment variables for Liu & Li's mapmaking routines in GDL
#
echo "Usage:  . ./LLmapmaking_setup.sh <7 directory names (see script)>"
# The dots and the space are important.

### SOURCE + compiled files
# No files will be modified in the following directories:

# directory with LLmapmaking .pro files
export LLMAP_PRO=$1

# top level directory containing  pro/
export WMAP_IDL_PRO=$2/pro

# top level directory containing directories of [directories of] .pro files 
export HEALPIX_TOP=$3
export HEALPIX_PRO=$3/fits:$3/misc:$3/zzz_external/cgis:$3/toolkit:$3/zzz_external/obsolete_astron

# top level directory containing .pro file directories
export IDLASTRO_TOP=$4
export IDLASTRO_PRO=$4/pro/fits:$4/pro/fits_table:$4/pro/fits_bintable:$4/pro/astro:$4/pro/misc:$4/pro/structure

# You need header files for the same version of GDL that you will run,
# in order to compile some routines and use the linkimage procedure.
# A clean way to do this is to install GDL from source.
export GDL_SRC=$5

# Some files and subdirectories will be created/modified in the following directories:
# root directory of LLmapmaking_GDL, i.e. the GDL patches on top of LLmapmaking 
export LLMAP_GDL=$6

### DATA
# top directory for input WMAP files and LLmapmaking output files
export LLMAP_DATA=$7



# If you already use GDL, then integrate the following parameters into
# the environment variable definitions in your existing shell startup file
# and into your GDL startup file, and comment them out here. 

# ${LLMAP_GDL}/pro is placed first in order that patched files override originals
# TODO: Does GDL respect path priority, i.e. highest at left?

# A single command is OK as long as you don't apply it repeatedly.
#export GDL_PATH=${LLMAP_GDL}/pro:${LLMAP_PRO}:${WMAP_IDL_PRO}:${HEALPIX_PRO}:${IDLASTRO_PRO}:${GDL_PATH}

if ! (echo ${GDL_PATH} |grep -q ${IDLASTRO_PRO}) ; then
    GDL_PATH=${IDLASTRO_PRO}:${GDL_PATH}
fi
if ! (echo ${GDL_PATH} |grep -q ${HEALPIX_PRO}) ; then
    GDL_PATH=${HEALPIX_PRO}:${GDL_PATH}
fi
if ! (echo ${GDL_PATH} |grep -q ${WMAP_IDL_PRO}) ; then
    GDL_PATH=${WMAP_IDL_PRO}:${GDL_PATH}
fi
if ! (echo ${GDL_PATH} |grep -q ${LLMAP_PRO}) ; then
    GDL_PATH=${LLMAP_PRO}:${GDL_PATH}
fi
if ! (echo ${GDL_PATH} |grep -q ${LLMAP_GDL}) ; then
    GDL_PATH=${LLMAP_GDL}/pro:${GDL_PATH}
fi

export GDL_STARTUP=${LLMAP_GDL}/pro/gdl_startup.pro
export GDL_LIB=${LLMAP_GDL}/lib

# You need to put the input files here. They are big and many.
# See e.g. http://cosmocoffee.info/viewtopic.php?p=4525#4525 
export LLMAP_TOD_DIR=${LLMAP_DATA}/WMAP3_TOD/ && mkdir -p ${LLMAP_TOD_DIR}
export LLMAP_WMAPDIR=${LLMAP_DATA}/WMAP3/  && mkdir -p ${LLMAP_WMAPDIR}

export LLMAP_WORKING_DIR=${LLMAP_DATA}/WMAP3_work/ && mkdir -p ${LLMAP_WORKING_DIR}

export LLMAP_PROCMASK=wmap_processing_mask_r9_3yr_v2.fits
ls -l ${LLMAP_WMAPDIR}/${PROCMASK} # check

if [ -d ${LLMAP_DATA} ] & [ -d ${LLMAP_TOD_DIR} ] & \
    [ -d ${LLMAP_WORKING_DIR} ] & [ -d ${LLMAP_WMAPDIR} ] \
    ; then
    echo Directories found. OK
    export LLMAP_TOD_BIN_DIR=${LLMAP_WORKING_DIR}/wmap_tod_bin/
    export LLMAP_OUTDIR=${LLMAP_WORKING_DIR}/mapmaking/
    mkdir -p ${LLMAP_TOD_BIN_DIR}
    mkdir -p ${LLMAP_OUTDIR}
    f_type=0
    case ${f_type} in
        0)
            f_str='no_f/'
            ;;
        1)
            f_str='f-safe/'
            ;;
        2)  
            f_str='f-m/'
            ;;
    esac

    export LLMAP_TOD_BIN_DIR_F=${LLMAP_TOD_BIN_DIR}/${f_str}
    export LLMAP_OUTDIR_F=${LLMAP_OUTDIR}/${f_str}
    export LLMAP_OUTDIR_MEDIAN_F=${LLMAP_OUTDIR}/${f_str}/'median_results/'

    mkdir -p ${LLMAP_TOD_BIN_DIR_F}
    mkdir -p ${LLMAP_OUTDIR_F}
    mkdir -p ${LLMAP_OUTDIR_MEDIAN_F}
    echo Directories exist and/or created. OK
else
    echo Directories not found. Please correct the script or create the directories.
fi

echo ""
echo "The following environment variables are presently set."
echo "You may have problems if any of them are wrong."
set|grep "^GDL\|^LLMAP"

