// mapxgen: core loop of Liu & Li''s mapmaking program, for use in GDL script
// converted to C++/GDL version by B. Roukema 2010-04-04

#include "envt.hpp"

using namespace std;

// TODO (low priority): Restore C++ template functionality for increased robustness.
BaseGDL* mapxgen_fun_notemplate( BaseGDL* map_a,
                                 BaseGDL* map_b ,
                                 BaseGDL* current_map,
                                 BaseGDL* current_nobs,
                                 BaseGDL* pa ,
                                 BaseGDL* pb ,
                                 BaseGDL* data ,
                                 BaseGDL* na ,
                                 BaseGDL* nb ,
                                 BaseGDL* axim_ar,
                                 BaseGDL* axim_br,
                                 BaseGDL* tod_size,
                                 BaseGDL* map_size,
                                 BaseGDL* nobs_flag
                                 )
{
  DFloatGDL* map_aC = static_cast<DFloatGDL*>( map_a);
  DFloatGDL* map_bC = static_cast<DFloatGDL*>( map_b);
  DFloatGDL* current_mapC = static_cast<DFloatGDL*>( current_map);
  DFloatGDL* current_nobsC = static_cast<DFloatGDL*>( current_nobs);

  DLongGDL* paC = static_cast<DLongGDL*>( pa);
  DLongGDL* pbC = static_cast<DLongGDL*>( pb);
  DFloatGDL* dataC = static_cast<DFloatGDL*>( data);
  DFloatGDL* naC = static_cast<DFloatGDL*>( na);
  DFloatGDL* nbC = static_cast<DFloatGDL*>( nb);
  DFloatGDL* axim_arC = static_cast<DFloatGDL*>( axim_ar);
  DFloatGDL* axim_brC = static_cast<DFloatGDL*>( axim_br);

  DLongGDL* tod_sizeC = static_cast<DLongGDL*>( tod_size);
  DLongGDL* map_sizeC = static_cast<DLongGDL*>( map_size);  /* not used */
  DLongGDL* nobs_flagC = static_cast<DLongGDL*>( nobs_flag);

  /* LOCAL */
  long i;

  if ((*nobs_flagC)[0] == 0){
    for (i=0; i< (*tod_sizeC)[0]; i++){ 
      (*current_mapC)[(*paC)[i]] += (*dataC)[i] * (*axim_arC)[0] + (*map_bC)[(*pbC)[i]];
      (*current_mapC)[(*pbC)[i]] += -((*dataC)[i] * (*axim_brC)[0]) + (*map_aC)[(*paC)[i]];
      (*current_nobsC)[(*paC)[i]]++ ;
      (*current_nobsC)[(*pbC)[i]]++ ;
      (*naC)[(*paC)[i]]++ ;
      (*nbC)[(*pbC)[i]]++ ;
    }
  }
  else{
    for (i=0; i< (*tod_sizeC)[0]; i++){ 
      (*current_mapC)[(*paC)[i]] += + (*dataC)[i] * (*axim_arC)[0] + (*map_bC)[(*pbC)[i]];
      (*current_mapC)[(*pbC)[i]] += - ((*dataC)[i] * (*axim_brC)[0]) + (*map_aC)[(*paC)[i]];
    }
  };

}


extern "C" BaseGDL* mapxgen_fun( EnvT* e)
{

  SizeT nParam=e->NParam();
  if (nParam != 14) {
    cout << "MAPXGEN ERROR: Improper Number of Variables" << endl;
    return new DLongGDL( -1);
  }

  BaseGDL* map_a = e->GetPar(0);
  BaseGDL* map_b = e->GetPar(1);
  BaseGDL* current_map = e->GetPar(2);
  BaseGDL* current_nobs = e->GetPar(3);
  BaseGDL* pa = e->GetPar(4);
  BaseGDL* pb = e->GetPar(5);
  BaseGDL* data = e->GetPar(6);
  BaseGDL* na = e->GetPar(7);
  BaseGDL* nb = e->GetPar(8);
  BaseGDL* axim_ar = e->GetPar(9);
  BaseGDL* axim_br = e->GetPar(10);
  BaseGDL* tod_size = e->GetPar(11); 
  BaseGDL* map_size = e->GetPar(12); 
  BaseGDL* nobs_flag = e->GetPar(13);

  return mapxgen_fun_notemplate(map_a, map_b, current_map,current_nobs,pa,pb,data,na,nb,  
                                axim_ar,axim_br,tod_size,map_size,nobs_flag);

  /* if there's an error */
  return new DLongGDL( -1);

}

