// swapxendianxinplace v0.1.1 
// (c) Boud Roukema, GNU General Public Licence version 2 or later 

#include "envt.hpp"
// This is provided by the GNU C compiler (gcc) version 2 or later.
#include <byteswap.h>

using namespace std;

template< typename T>
BaseGDL* swapxendianxinplace_fun_template( BaseGDL* p0)
{
  T* p0C = static_cast<T*>( p0);
  T* res = new T( p0C->Dim(), BaseGDL::NOZERO);
  SizeT nEl = p0->N_Elements();

  for( SizeT i=0; i<nEl; ++i)
    {
      if ( p0->Type() == DOUBLE){
        (*p0C)[i] = bswap_64_double( ((*p0C)[i]) ) ;
      }else if( p0->Type() == FLOAT){
        (*p0C)[i] = bswap_32_double( ((*p0C)[i]) ) ;
      };
    };
}

extern "C" BaseGDL* swapxendianxinplace_fun( EnvT* e)
{

  SizeT nParam=e->NParam();
  if (nParam != 1) {
    cout << "SWAPXENDIANXINPLACE: Improper Number of Variables" << endl;
    return new DLongGDL( -1);
  }

  BaseGDL* p0 = e->GetPar( 0);//, "SWAPXENDIANXINPLACE64D");

  if( p0->Type() == DOUBLE){
    return swapxendianxinplace_fun_template< DDoubleGDL>( p0);
  }else if (p0->Type() == FLOAT){
    return swapxendianxinplace_fun_template< DFloatGDL>( p0);
  }else
    {
      return new DLongGDL( -1);
    }
}


// 64-bit floating-point wrapper for the GNU C compiler (gcc) version
// 2 or later endianness swapper for 64-bit integers.
// (c) Boud Roukema, GNU General Public Licence version 2 or later 
double bswap_64_double(double x){
  unsigned long long i;
  double y;
  i= bswap_64(  *((unsigned long long*)(&x)) );
  y=  *( (double*)(&i) );
  return y;
};

// 32-bit floating-point wrapper for the GNU C compiler (gcc) version
// 2 or later endianness swapper for 32-bit integers.
// (c) Boud Roukema, GNU General Public Licence version 2 or later 
float bswap_32_double(float x){
  unsigned long i;
  float y;
  i= bswap_32(  *((unsigned long*)(&x)) );
  y=  *( (float*)(&i) );
  return y;
};
